% FORCES PRO : A customized generator for fast optimization solvers.
%
% Generatces code for a simple MPC problem:
% 
%  min   xN'*P*xN + sum_{i=1}^{N-1} xi'*Q*xi + ui'*R*ui
% xi,ui
%       s.t. x1 = x
%            x_i+1 = A*xi + B*ui  for i = 1...N-1
%            xmin <= xi <= xmax   for i = 1...N
%            umin <= ui <= umax   for i = 1...N-1
%
% and P is solution of Ricatti eqn. from LQR problem
%
% Copyright (C) embotech GmbH, Zurich, Switzerland, 2015.
clc


%% system
nx = 2;
nu = 1;
A = [   0.7115  -0.4345; ...
        0.4345  0.8853  ];
B = [   0.2173; ...
        0.0573  ];


%% MPC setup
N = 10 + 1;
Q = [   10  0; ...
        0   10  ];
R = 1;
[~,P] = dlqr(A,B,Q,R);
umin = -5;       umax = 5;
xmin = [-inf; 0]; xmax = [inf; inf];   

%% FORCES multistage form
% assume variable ordering zi = [ui, xi+1] for i=1...N-1

stages = MultistageProblem(N);
for i = 1:N
    
        % dimension
        stages(i).dims.n = nx+nu;   % number of stage variables
        stages(i).dims.r = nx;      % number of equality constraints        
        stages(i).dims.l = nu;      % number of lower bounds (MODIFY THIS!)
        stages(i).dims.u = nu;      % number of upper bounds
        
        % cost
        stages(i).cost.H = blkdiag(R,Q);
        % ADD SOME CODE HERE!
        stages(i).cost.f = zeros(nx+nu,1);
        
        % lower bounds - MODIFY THIS!
        stages(i).ineq.b.lbidx = 1; % lower bound acts on these indices
        stages(i).ineq.b.lb = umin; % lower bound for this stage variable
        
        % upper bounds
        stages(i).ineq.b.ubidx = 1; % upper bound acts on these indices
        stages(i).ineq.b.ub = umax; % upper bound for this stage variable
        
        % ADD QUADRATIC CONSTRAINTS HERE
        
        % equality constraints
        if( i < N )
            stages(i).eq.C = [zeros(nx,nu), A];
        end
        if( i>1 )
            stages(i).eq.c = zeros(nx,1);
        end
        stages(i).eq.D = [B, -eye(nx)];
        
end
params(1) = newParam('minusA_times_x0',1,'eq.c'); % RHS of first eq. constr. is a parameter: z1=-A*x0

%% define outputs of the solver
outputs = newOutput('u1',1,1:nu);

%% solver settings
codeoptions = getOptions('forcesproController');
codeoptions.optlevel = 3; %0: The optimization is disabled, but it takes less time to generate the mex-file.
codeoptions.overwrite = 1; %1: Always overwrite existing code.
codeoptions.printlevel = 0; %0: no printing

codeoptions.solvemethod = 'PDIP';
codeoptions.maxit       = 500;
% ADD MORE SOLVER SETTINGS HERE!


%% generate code
generateCode(stages,params,codeoptions,outputs);

