% Run closed-loop simulation with FORCES Pro.
% Copyright (C) embotech GmbH, Zurich, Switzerland, 2015.
clc
clear

%System Definition
A=[0.7115 -0.4345; 0.4345 0.8853];
B=[0.2173;0.0573];
C=[0 1];
[nx,nu]=size(B);

%Controller Tuning Parameters
Q=10*eye(2);
R=1;
N=10;
[K,P]=dlqr(A,B,Q,R);

%Input Constraints
umin=-5;
umax=5;

%State Constraints (-/+inf for none)
xmin=[-inf;-inf];
xmax=[inf;inf];
%xmin(2)=0; %state constraint


%% Solver Selection
solver='quadprog';
%solver='myforcespro';

if strcmp(solver,'myforcespro')
    myforcespro; % generate FORCES Pro controller 
end


%% Simulation

%setup
x0=[0;6];
T=3;
Ts=0.1;
t=[0:Ts:T];
nT=length(t);

%pre-allocate
xvec=nan(nT,nx);
uvec=nan(size(t));
timings=nan(nT-1,1);
iterations=nan(nT-1,1);

z0=zeros(N*nu,1); %starting poing for solvers

%loop
xcur=x0; %initial state of plant
for k=1:nT-1
    xvec(k,:)=xcur;
    [uvec(k),U,iter,time]=mympc(A,B,Q,R,P,N,umin,umax,xmin,xmax,xcur,solver,z0);
    xcur=A*xcur+B*uvec(k);
    
    timings(k)=time;
    iterations(k)=iter;
end
xvec(nT,:)=xcur;

fprintf('max_time: %1.4e s\n',max(timings(2:end)));
fprintf('max_iter: %d\n',max(iterations(2:end)));

%plot
figure(1);
subplot(2,1,1)
plot(t,xvec(:,1), t,xvec(:,2),'LineWidth',2);
xlim([t(1) t(end)])
grid on;
xlabel('t')
ylabel('state x')
legend('x1','x2');
subplot(2,1,2)
stairs(t,uvec,'LineWidth',2);
hold on;
plot([t(1) t(end)], [umax umax]', 'r--','LineWidth',2); plot([t(1) t(end)], [umin umin]', 'r--','LineWidth',2);
xlim([t(1) t(end)])
grid on;
xlabel('t')
ylabel('input u')
